<?php

class Sat_AI_Handler
{
    private $errors;
    private $success;
    private $notice;

    public function __construct()
    {
        add_action('admin_init', array($this, "handle_form"));
        $this->errors = array();
        $this->success = array();
        $this->notice = array();
    }

    private function sat_ai_send_contact(){
        return json_decode('{
            "name": "sat_ai_send_contact",
            "description": "Startet und sendet eine Kontaktanfrage, Frage die E-Mail oder die Telefonnumer an, da ansonsten eine Anfrage nicht möglich ist",
            "parameters": {
                "type": "object",
                "properties": {
                    "reason": {
                        "type": "string",
                        "description": "Der Anfragegrund"
                    },
                    "email": {
                        "type": "string",
                        "description": "Die Email-Adresse des Users, für Kontaktanfrage notwendig"
                    },
                    "tel": {
                        "type": "string",
                        "description": "Die Telefonnummer des Users, für rückruf notwendig"
                    },
                    "name": {
                        "type": "string",
                        "description": "Der Name des Benutzers"
                    }
                },
                "required": [
                    "tel",
                    "email"
                ]
            }
        }', true);
    }

    public function handle_form()
    {

        // Check if we are needed
        if (!isset($_POST['submit_setting']) && !isset($_POST['task'])) {
            return;
        }

        // Bail out if not authorized
        if (!wp_verify_nonce($_POST['_wpnonce'], 'sat-ai-settings')) {
            wp_die(esc_html(__('You are not authorized!', "sat-ai")));
        }

        if (!current_user_can('manage_options')) {
            wp_die(esc_html(__('Insufficent permission!', 'sat-ai')));
        }

        $this->errors = array();
        $this->success = array();
        $this->notice = array();
        $page_url = admin_url('admin.php?page=sat-ai');


        // Get Input and sanitize
        $task = isset($_POST['task']) ? sanitize_text_field($_POST['task']) : "";

        // Do the task
        switch ($task) {
            case "set_api_settings":
                $this->save_api_settings();
                break;
            case "set_openai_general_setting":
                $this->save_general_settings();
                $page_url .= "&action=general";
                break;
            default:
                break;
        }

        // Error Output gen
        if ($this->errors) {
            $first_error = reset($this->errors);
            $redirect_to = add_query_arg(array('error' => urlencode($first_error)), $page_url);
            wp_safe_redirect($redirect_to);
            exit;
        }
        if ($this->success) {
            $first_success = reset($this->success);
            $redirect_to = add_query_arg(array('success' => urlencode($first_success)), $page_url);
            if ($this->notice) {
                $first_notice = reset($this->notice);
                $redirect_to = add_query_arg(array('notice' => urlencode($first_notice)), $redirect_to);
            }
            wp_safe_redirect($redirect_to);
            exit;
        }
    }

    private function save_api_settings()
    {
        // Nonce does not need to be verified, since it was already veryfied by handle_form() 
        // Sanitize and isset Check
        $submitted_openai_api_key = sanitize_text_field($_POST['openai_key'] ?? "");
        $dsgvo_setting = sanitize_text_field($_POST["sat_dsgvo_setting"] ?? "");
        $sat_ai_tool = sanitize_text_field($_POST["sat_dsgvo_setting"] ?? "");
        $submitted_openai_assistant_id = sanitize_text_field($_POST["gpt_assistant"] ?? "");
        $sat_ai_chathistory_setting = sanitize_text_field($_POST["sat_chat_history_setting"] ?? "");

        // Check if obscured string was sent if so exchange it with real db value
        $openai_api_key = get_option("sat_openai_key");
        $obscured_openai_api_key = sat_ai_obscure_sensitive_strings($openai_api_key);
        if ($submitted_openai_api_key !== $obscured_openai_api_key) {
            $openai_api_key = $submitted_openai_api_key;
        }

        // Check if obscured string was sent if so exchange it with real db value
        $openai_assistant_id = get_option("sat_openai_assistant");
        $obscured_openai_assistant_id = sat_ai_obscure_sensitive_strings($openai_assistant_id);
        if ($obscured_openai_assistant_id !== $submitted_openai_assistant_id) {
            $openai_assistant_id = $submitted_openai_assistant_id;
        }

        // Bail out and do not save incomplete settings
        if ($openai_api_key == "" || $dsgvo_setting == "" || $openai_assistant_id == "" || $sat_ai_chathistory_setting == "") {
            $this->errors[] = __("The API settings could not be updated.", "sat-ai");
            return;
        }

        // Translate DSGVO on/off for DB
        if ($dsgvo_setting == "On") {
            $dsgvo_setting = 1;
        } else {
            $dsgvo_setting = 0;
        }
        // Translate Chathistory setting on/off for DB
        if ($sat_ai_chathistory_setting == "On") {
            $sat_ai_chathistory_setting = 1;
        } else {
            $sat_ai_chathistory_setting = 0;
        }

        // Selected API no further translation/checking
        // openai key does not require further checking/sanitizing
        // Save settings in DB
        $openai_assistant_save_result = update_option('sat_openai_assistant', $openai_assistant_id);
        $openai_api_key_save_result = update_option('sat_openai_key', $openai_api_key);
        $dsgvo_setting_save_result = update_option('sat_openai_dsgvo_behaviour', $dsgvo_setting);
        $sat_ai_chathistory_save_result = update_option("sat_openai_chathistory_behaviour", $sat_ai_chathistory_setting);

        $modifie_args = array();
        // retrieve fields for updating assistant with local values
        $openai_modell = get_option("sat_openai_modell");
        if ($openai_modell) {
            $modifie_args["model"] = $openai_modell;
        }
        $role = get_sat_ai_openai_role();
        if ($role) {
            $modifie_args["instructions"] = $role;
        }
        $sat_ai_tool = get_option("sat_ai_tool");
        $openai_tool_array = json_decode("", true);
        if ($sat_ai_tool == 1) {
            $openai_tool_array = $this->sat_ai_send_contact();
        }
        if ($openai_tool_array !== null) {
            $tools = array();

            $tools[] = array(
                "type" => "function",
                "function" => $openai_tool_array
            );

            $modifie_args["tools"] = $tools;
        } else {
            $modifie_args["tools"] = array();
        }

        if (!empty($modifie_args)) {
            $modifie_args_save_result = sat_modify_assistant_parameters($modifie_args);
        }

        if ($openai_api_key_save_result || $dsgvo_setting_save_result || $openai_assistant_save_result || $sat_ai_chathistory_save_result) {
            $this->success[] = __("The API settings were successfully updated.", "sat-ai");
        } else {
            $this->errors[] = __("The API settings could not be updated.", "sat-ai");
        }
        return;
    }

    private function save_general_settings()
    {
        // Nonce does not need to be verified, since it was already veryfied by handle_form() 
        // Collect common setting Values and check if set and sanitize them
        $openai_modell = sanitize_text_field($_POST["gpt_modell"] ??  "");
        $openai_role = intval(sanitize_textarea_field($_POST["sat_chatbot_rolle"] ?? ""));

        $allowed_html = sat_ai_allowed_html();

        $openai_start_message = wp_kses($_POST["chatbot_first_message"] ?? "", $allowed_html);

        $openai_chat_request_limit = intval(sanitize_text_field($_POST["chatbot_chat_limit"] ?? "0"));
        // 0 == Unlimited

        $openai_last_chat_message = wp_kses($_POST["chatbot_last_message"] ?? "", $allowed_html);
        // Allow HTML + JS -> for example allow to send a link to contact page

        $sat_ai_tool = sanitize_text_field($_POST["sat_ai_tool"] ?? "");
        $openai_receiver_mail = sanitize_email($_POST["openai_receiver_mail"] ?? "");

        if ($openai_modell == "" || $openai_role == "" || $openai_role == 0 || $openai_modell == 0) {
            $this->errors[] = sprintf(
                /* translators:
                    $s: Translation for "GPT-Modell"
                    $s: Translation for "Chatbot Rolle" */
                __('The required fields %1$s and %2$s are unset.', "sat-ai"),
                _x("GPT-Model", "field-name", "sat-ai"), _x("Chatbot Role", "field-name", "sat-ai")
            );
            return;
        }
        if ($sat_ai_tool == "On") {
            $sat_ai_tool = 1;
        } else {
            $sat_ai_tool = 0;
        }
        // Updating Values
        $openai_modell_save_result = update_option('sat_openai_modell', $openai_modell);
        $openai_role_save_result = update_option('sat_openai_role', $openai_role);
        $start_message_save_result = update_option('sat_chatbot_first_message', $openai_start_message);
        $last_message_save_result = update_option("sat_chat_bot_last_message", $openai_last_chat_message);
        $message_request_limit_save_result = update_option("sat_openai_chat_limit", $openai_chat_request_limit);
        $openai_tool_save_result = update_option("sat_ai_tool", $sat_ai_tool);
        $openai_receiver_mail_save_result = update_option('sat_openai_receiver_mail', $openai_receiver_mail);
        $tools = array();

        $modifie_args = array(
            "model" => $openai_modell,
        );
        $role_text = get_sat_ai_openai_role();
        if (!empty($role_text)) {
            $modifie_args["instructions"] = $role_text;
        }

        if ($sat_ai_tool == 1) {
            $tools[] = array(
                // required fields
                "type" => "function",
                "function" => $this->sat_ai_send_contact()
            );
        }
        $modifie_args["tools"] = $tools;

        $assistant_modification_result = sat_modify_assistant_parameters($modifie_args);
        if (
            $openai_modell_save_result ||
            $openai_role_save_result ||
            $start_message_save_result ||
            $last_message_save_result ||
            $message_request_limit_save_result ||
            $openai_tool_save_result ||
            $openai_tool_save_result ||
            $assistant_modification_result
        ) {
            $this->success[] = __("The general settings were successfully updated.", "sat-ai");
        } else {
            $this->errors[] = __("The general setting could not be updated.", "sat-ai");
        }
    }
}

new Sat_AI_Handler();