<?php

/**
 * Sends a request to gpt Assistants API
 *
 * @param string    $prompt
 *
 * @param string    $pageContext
 *
 * @param string    $embeddingcontext
 *
 * @return array{messages:array{string}, events:array{string}}    An array of messages and events
 */
function sat_ai_doAssistantChat(string $prompt = "", string $pagecontext = "", string $embeddingcontext = "")
{
    // SESSION
    if (session_status() == PHP_SESSION_NONE) {
        sat_ai_start_session();
    }

    // Anfragelimit erreicht
    if (sat_ai_checkRequestLimit()) {
        return array(
            "messages" => [
                array(
                    "message" => array(
                        "value" => stripslashes(get_option("sat_chat_bot_last_message", __("User request limit reached", "sat-ai"))),
                        "annotations" => "",
                    ),
                    "role" => "assistant"
                ),
            ],
            "events" => array()
        );
    }
    // Frontend Assistant gesetzt?
    $assistant_id = get_option("sat_openai_assistant");
    if (empty($assistant_id) || $assistant_id == "0" || $assistant_id === false) {
        return array(
            "messages" => [
                array(
                    "message" => array(
                        "value" => __("There is no assistant configured.", "sat-ai"),
                        "annotations" => "",
                    ),
                    "role" => "assistant"
                ),
            ],
            "events" => array()
        );
    }
    try {
        $client = sat_ai_get_client();
        // TODO for later Usage
        // If custom Functions overwrite
        // ensures that chat and Thread is running and existing
        if (!isset($_SESSION["sat_chatThreadID"]) || empty($_SESSION["sat_chatThreadID"])) {
            $_SESSION["sat_chatThreadID"] = $client->threads()->create([])->id;
        }
        $thread_id = $_SESSION["sat_chatThreadID"];
        if (!isset($_SESSION["sat_chatID"]) || empty($_SESSION["sat_chatID"])) {

            $_SESSION["sat_chatID"] = sat_ai_start_chat_assistant($_SESSION["sat_chatThreadID"]);
        }
        // Create Message

        $created_message = $client->threads()->messages()->create(
            $thread_id,
            [
                'role' => 'user',
                'content' => $prompt
            ]
        );
        // for display usage
        $_SESSION["sat_ai_all_messages"][] = array(
            'role' => 'user', 'content' => $prompt
        );
        $parameters = sat_get_assistant_parameters($client, $assistant_id);
        // Check if the assistant could be fetched
        if (isset($parameters[0]) && $parameters[0] === false) {
            return array(
                "messages" => [
                    array(
                        "message" => array(
                            "value" => $parameters[1],
                            "annotations" => "",
                        ),
                        "role" => "assistant"
                    ),
                ],
                "events" => array()
            );
        }
        // Run it
        $run_message = $client->threads()->runs()->create(
            threadId: $thread_id,
            //paramerters: $parameters, // only needed if the parameters should change by session instead globally
            // followed parameters is bare minimum - gpt fetches any settings through assistant
            parameters: array(
                'assistant_id' => $assistant_id
            )
        );

        // Check if completed

        $queue_status = $run_message->status;
        $run_id = $run_message->id;
        $events_to_raise = array();
        while ($queue_status == "queued" || $queue_status == "in_progress" || $queue_status == "requires_action") {
            // 100 Milliseconds -- max. 10 Updates per second (reduces server and api load)
            usleep(100000);
            // Updating Queue Status
            $running = $client->threads()->runs()->retrieve(
                threadId: $thread_id,
                runId: $run_id,
            );
            $queue_status = $running->status;
            if ($queue_status == "requires_action") {
                if ($running->requiredAction->type == "submit_tool_outputs") {

                    $tool_call_result_obj = sat_ai_doToolSubmit($running);
                    $tool_call_result = $tool_call_result_obj["tool_result"];
                    $events_to_raise = array_merge($events_to_raise, $tool_call_result_obj["tool_events"]);

                    $client->threads()->runs()->submitToolOutputs(
                        threadId: $thread_id,
                        runId: $run_id,
                        parameters: [
                            "tool_outputs" => $tool_call_result
                        ]
                    );
                }
            }
        }

        if ($queue_status == "completed") {
            // Saving Prompt

            $question_obj = array(
                'chatID' => $_SESSION['sat_chatID'],
                'role' => 'user',
                'message' => $prompt,
                'tokens' => $running->usage->promptTokens,
            );

            sat_ai_save_chatmessage($question_obj);
            $new_messages = $client->threads()->messages()->list(
                $thread_id,
                [
                    // 'limit' => 10,
                    'before' => $created_message->id
                    // Messages are ordered as followed:
                    // latest, ..., current user msg, ..., old messages
                ]
            );
            $result_obj = array(
                "messages" => array(),
                "events" => $events_to_raise
            );

            // QoL-Update - allow certain HTML-Elements to improve Visual output
            $allowed_html = sat_ai_allowed_html();

            foreach ($new_messages->data as $message_obj) {

                $sanitized_message_obj = (object) array(
                    "value" => wp_kses($message_obj->content[0]->text->value, $allowed_html),
                    "annotations" => $message_obj->content[0]->text->annotations
                );
                $result_obj["messages"][] = array(
                    "role" => $message_obj->role,
                    "message" => $sanitized_message_obj // Beware for Break Chance since possible other content types/multiple...
                );

                $_SESSION["sat_ai_all_messages"][] = array(
                    "role" => $message_obj->role,
                    'content' => $sanitized_message_obj->value
                );
                $answer_obj = array(
                    'chatID' => $_SESSION['sat_chatID'],
                    'role' => 'assistant',
                    'message' => $sanitized_message_obj->value,
                    'tokens' => $running->usage->completionTokens,
                );
                sat_ai_save_chatmessage($answer_obj);
            }

            return $result_obj;
        }
    } catch (OpenAI\Exceptions\ErrorException $exc) {
        if (str_contains($exc->getErrorMessage(), "No assistant found with id")) {
            return array(
                "messages" => [
                    array(
                        "message" => array(
                            "value" => __("Error: Please check if the configured Assistant ID is valid.", "sat-ai"),
                            "annotations" => "",
                        ),
                        "role" => "assistant"
                    ),
                ],
                "events" => array()
            );
        }
        
        return false;
    } catch (OpenAI\Exceptions\TransporterException $exc) {
        if (str_contains($exc->getMessage(), "Idle timeout reached for")) {
            return array(
                "messages" => [
                    array(
                        "message" => array(
                            "value" => __("Error: Your API request timed out. Please try again in a few minutes.", "sat-ai"), // TODO Rephrase it?
                            "annotations" => "",
                        ),
                        "role" => "assistant"
                    ),
                ],
                "events" => array()
            );
        } elseif (str_contains($exc->getMessage(), "Can't add messages to")) {
            // Request can not be resolved because it is running and needs to be aborted
            $last_runs = $client->threads()->runs()->list(
                threadId: $thread_id,
                parameters: [
                    'limit' => 10,
                ],
            );
            $client->threads()->runs()->cancel(
                threadId: $thread_id,
                runId: $last_runs->lastId
            );
            // retry to fulfill request 
            return sat_ai_doAssistantChat($prompt, $pagecontext, $embeddingcontext);
        }
    }
    die();
}

function sat_get_assistant_parameters($client, $assistant_id)
{
    // Fail early, before assistant is retrieved
    $ai_role = get_sat_ai_openai_role();
    if ($ai_role === false || $ai_role == "0") {
        return array(false, __("Error: No role configured.", "sat-ai"));
    }
    $ai_modell = get_option("sat_openai_modell");
    if ($ai_modell === false || $ai_modell == "0" || empty($ai_modell)) {
        return array(false, __("Error: No model selected.", "sat-ai"));
    }
    return true;
}

function sat_modify_assistant_parameters($parameters)
{
    $client = sat_ai_get_client();
    if (!$client) {
        return false;
    }
    $assistant_id = get_option("sat_openai_assistant");
    if ($assistant_id === false || $assistant_id == "") {
        return false;
    }

    try {
        $assistant = $client->assistants()->retrieve($assistant_id);
        $default_parameters = array(
            "name" => $assistant->name === null ? "" : $assistant->name, 
            "description" => $assistant->description === null ? "" : $assistant->description,
            "model" => $assistant->model ? "" : $assistant->model,
            "instructions" => $assistant->instructions === null ? "" : $assistant->instructions,
            "tools" => $assistant->tools === null ? "" : $assistant->tools,
        );
        $modified_parameters = wp_parse_args($parameters, $default_parameters);
        $client->assistants()->modify($assistant_id, $modified_parameters);
        return true;
    } catch (OpenAI\Exceptions\ErrorException $e) {
        return false;
    }
}

function sat_ai_start_chat_assistant($thread_id)
{
    $sat_ai_chathistory_setting = get_option("sat_openai_chathistory_behaviour");
    if ($sat_ai_chathistory_setting == 0 || $sat_ai_chathistory_setting == false) {
        return $thread_id;
    }
    global $wpdb;
    $table = $wpdb->prefix . "sat_chathistorie";

    $args = array(
        'started' => gmdate('Y-m-d H:i:s'),
        'sessionID' => session_id(),
        "ip" => $_SERVER['REMOTE_ADDR'],
        'thread_id' => $thread_id,
    );

    $wpdb->insert($table, $args); //db call ok

    $chatID = $wpdb->insert_id;
    return $chatID;
}

function sat_ai_doToolSubmit($thread)
{
    $toolcalls = $thread->requiredAction->submitToolOutputs;
    $tool_call_result = array(
        "tool_result" => array(),
        "tool_events" => array()
    );
    foreach ($toolcalls as $toolcall) {
        $tool = $toolcall[0];
        $tool_id = $tool->id;
        $type = $tool->type;
        $tool_result = "failed";
        if ($type == "function") {
            $function_name = $tool->function->name;
            if (function_exists($function_name)) {
                $function_args = json_decode($tool->function->arguments);
                $result_obj = call_user_func($function_name, $function_args);
                $result = $result_obj["result"];

                if ($result) {
                    $tool_result = "success";
                }
            }
            $tool_obj = array(
                'chatID' => $_SESSION['sat_chatID'],
                'role' => 'assistant',
                'message' => sprintf(
                    /* translators:
                    $s: Name of the function"
                    $s: result of the function - failed | success" */
                    __('Function call: %1$s - %2$s', "sat-ai"),
                    $function_name,
                    $tool_result
                ),
                'tokens' => "",
            );

            sat_ai_save_chatmessage($tool_obj);

            $tool_call_result["tool_result"][] = array(
                "tool_call_id" => $tool_id,
                "output" => $tool_result,
            );
            $event_name = sat_ai_get_tool_event_name($function_name);
            if ($event_name) {
                $tool_call_result["tool_events"][] = $event_name;
            }
        }
    }
    return $tool_call_result;
}
