<?php

/**
 * Returns a list of modells
 *
 *
 * @return object
 */
function sat_ai_get_models()
{
    $client = sat_ai_get_client();
    if (!$client) {
        return false;
    }
    try {
        $models = $client->models()->list();
        return $models;
    } catch (OpenAI\Exceptions\ErrorException $exc) {
        return false;
    }
}

/**
 * Creates an Openai Client
 *
 *
 * @return OpenAI::client
 */
function sat_ai_get_client()
{
    $api_key = get_option("sat_openai_key");
    if (!$api_key) {
        return false;
    }

    $client = OpenAI::client($api_key);

    return $client;
}

function sat_ai_checkRequestLimit()
{
    $ai_chat_limit = get_option("sat_openai_chat_limit", 0);
    if ($ai_chat_limit != 0 && intdiv(count($_SESSION['sat_ai_all_messages']), 2) >= $ai_chat_limit) {
        return true;
    }
    return false;
}

/**
 * Sends an email
 *
 * @param array   $args
 *
 * @return array(
 *      bool result,
 *      string message
 * )
 */
function sat_ai_send_contact($args)
{
    $default = array(
        "reason" => _x('Contact request', "default-email-request-header", "sat-ai"),
        "email" => '',
        "tel" => '',
        "name" => '',
    );
    $kontakt_informationen = wp_parse_args($args, $default);

    // Check if Mail/Tel is in User Messages
    $mail_in_messages = false;
    $tel_in_messages = false;
    foreach ($_SESSION["sat_ai_all_messages"] as $msg) {
        if ($msg['role'] == 'user') {
            if (str_contains($msg['content'], $kontakt_informationen['email'])) {
                $mail_in_messages = true;
            }
            if (str_contains($msg['content'], $kontakt_informationen['tel'])) {
                $tel_in_messages = true;
            }
        }
    }
    // Check if GPT created them
    if ($mail_in_messages == false && $tel_in_messages == false) {
        return array(
            "result" => false,
            "message" => __("Your contact request could not be submitted. Please revalidate your entered contact information.", "sat-ai")
        );
    }

    // Check if Valid Mail or Tel

    $ai_receiver_mail = get_option("sat_openai_receiver_mail");
    // if invalid Mail/Tel
    if ($ai_receiver_mail == false || empty($ai_receiver_mail)) {
        return array(
            "result" => false,
            "message" => __("Your contact request could not be submitted.", "sat-ai")
        );
    }

    $has_email = false;
    if (is_email(sanitize_email($kontakt_informationen['email']))) {
        $has_email = true;
    }

    // it will only validate the given number corresponds to a specific format, it can not check if the number really existst (will match many number formats, not all)
    $phone_number_validation_regex = "/^[+]?(?=(?:[^\dx]*\d){7})(?:\(\d+(?:\.\d+)?\)|\d+(?:\.\d+)?)(?:[\/ -]?(?:\(\d+(?:\.\d+)?\)|\d+(?:\.\d+)?))*(?:[ ]?(?:x|ext)\.?[ ]?\d{1,5})?$/";
    $has_phone_number = false;

    if (preg_match($phone_number_validation_regex, $kontakt_informationen['tel'])) {
        $has_phone_number = true;
    }

    if (($has_phone_number == false || $tel_in_messages == false) && ($has_email == false || $mail_in_messages == false)) {
        return array(
            "result" => false,
            "message" => __("Your contact request could not be submitted. Please revalidate your entered contact information.", "sat-ai")
        );
    }
    // 'message' => sprintf(
    //     /* translators:
    //     $s: Name of the function"
    //     $s: result of the function - failed | success" */
    //     __('Function call: %1$s - %2$s', "sat-ai"),
    //     $function_name,
    //     $tool_result
    // ),
    $message = sprintf(
        /* translators:
        $s: reason for this contact request - migth be blank
        $s: Name of the person who started the request - migth be blank
        $s: E-mail Address of the person who started the request - migth be blank
        $s: phone number of the person who started the request - migth be blank 
        */
        esc_html_x('Reason for contact: %1$s
Name: %2$s
E-Mail: %3$s
Phone: %4$s

Chathistory:
', "contact-form-template", "sat-ai"),
        $kontakt_informationen['reason'],
        $kontakt_informationen['name'],
        $kontakt_informationen['email'],
        $kontakt_informationen['tel']
    );

    // add chat to message
    foreach ($_SESSION["sat_ai_all_messages"] as $msg) {
        $message .= $msg['role'] . ": " . $msg['content'] . "\r\n\r\n"; // Double Linebreak to increase read ability between user and bot
    }
    
    if (wp_mail($ai_receiver_mail, "Sat-AI: " . $kontakt_informationen['reason'], $message)) {
        return array(
            "result" => true,
            "message" => __("Your contact request was successfully submitted. A customer service agent will soon get in touch with you.", "sat-ai")
        );
    } else {
        return array(
            "result" => false,
            "message" => __("Your contact request could not be submitted. Please try again in a few minutes.", "sat-ai")
        );
    }
}

/**
 * Starts the php session to track the chat
 *
 */
function sat_ai_start_session()
{
    if (session_status() == PHP_SESSION_NONE) {
        session_start([
            "use_only_cookies" => 1,
            "cookie_lifetime" => 0,
            "cookie_secure" => 1,
            "cookie_httponly" => 1,
        ]);
    }

    $first_message = get_option("sat_chatbot_first_message");
    $first_message = $first_message != false ? stripslashes($first_message) : __("How can I help you?", "sat-ai");

    if (!isset($_SESSION["sat_ai_all_messages"]) || empty($_SESSION["sat_ai_all_messages"])) {
        $_SESSION["sat_ai_message"] = array();
    }

    if (!isset($_SESSION["sat_ai_all_messages"]) || empty($_SESSION["sat_ai_all_messages"])) {
        $_SESSION["sat_ai_all_messages"] = array(
            array(
                "role" => "assistant",
                "content" => $first_message,
            ),
        );
    }
}

/**
 * Creates the chat table
 *
 */
function sat_ai_create_chat_table()
{
    global $wpdb;
    $table = $wpdb->prefix . "sat_chathistorie";
    $charset_collate = $wpdb->get_charset_collate();

    $query = "CREATE TABLE $table (
        `chatID` INT UNSIGNED NOT NULL AUTO_INCREMENT,
        `started` DATETIME NOT NULL DEFAULT (CURRENT_TIMESTAMP),
        `sessionID` VARCHAR(255),
        `ip` VARCHAR(255),
        `thread_id` VARCHAR(255),
        PRIMARY KEY (chatID)) $charset_collate;";

    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($query);
}
/**
 * Creates the message table, which is tied to the chat table
 *
 */
function sat_ai_create_message_table()
{
    global $wpdb;
    $table = $wpdb->prefix . "sat_message_historie";
    $charset_collate = $wpdb->get_charset_collate();

    $query = "CREATE TABLE $table (
        `messageID` INT UNSIGNED NOT NULL AUTO_INCREMENT,
        `chatID` INT UNSIGNED NOT NULL,
        `role` ENUM('assistant', 'user', 'system', 'embedding') NOT NULL,
        `message` TEXT NOT NULL,
        `tokens` INT,
        `message_time` DATETIME NOT NULL DEFAULT (CURRENT_TIMESTAMP),
        PRIMARY KEY (messageID)) $charset_collate;";

    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($query);
}

/**
 * Save a chatmessage to the database
 *
 * @param array   $args
 *
 * @return int | None
 */
function sat_ai_save_chatmessage($args)
{
    $sat_ai_chathistory_setting = get_option("sat_openai_chathistory_behaviour");
    if($sat_ai_chathistory_setting == 0 || $sat_ai_chathistory_setting == false){
        return 0;
    }
    global $wpdb;
    $table = $wpdb->prefix . "sat_message_historie";

    $defaults = array(
        'messageID' => '',
        'chatID' => '',
        'role' => '',
        'message' => '',
        'tokens' => '',
        'message_time' => gmdate('Y-m-d H:i:s')
    );

    $args = wp_parse_args($args, $defaults);

    if ($wpdb->insert($table, $args)) { //db call ok
        return $wpdb->insert_id;
    }
}

/**
 * Get all Chat Verlauf
 * @param array $args {
 *  $args = [
 *      'orderby'   => (string) DB Column of Table sat_chathistorie
 *      'order'     => (string), (Default DESC if not equal ASC, asc, DESC, desc)
 *  ]
 *
 * @return array
 */
function sat_ai_get_all_Chat_Verlauf($args = array())
{
    global $wpdb;

    $defaults = array(
        'orderby'    => 'chatID',
        'order'      => 'DESC',
    );

    $args      = wp_parse_args($args, $defaults);
    $cache_key = 'Chat Verlauf-all';
    $items     = wp_cache_get($cache_key, 'sat-ai');
    if (!in_array($args["order"], array("asc", "ASC", "desc", "DESC"))) {
        $args["order"] = "DESC";
    }
    if (false === $items) {
        $items = $wpdb->get_results($wpdb->prepare("SELECT * FROM " . $wpdb->prefix . "sat_chathistorie ORDER BY %i ", $args['orderby']) . " " . $args["order"]); //db call ok
        wp_cache_set($cache_key, $items, 'sat-ai');
    }

    return $items;
}

/**
 * Fetch a single Chat Verlauf from database
 *
 * @param int   $id
 *
 * @return array
 */
function sat_ai_get_Chat_Verlauf($id = 0)
{
    global $wpdb;
    $cache_key = 'Chat Verlauf-' . $id;
    $items     = wp_cache_get($cache_key, 'sat-ai');
    if (false === $items) {
        $items = $wpdb->get_results($wpdb->prepare('SELECT * FROM ' . $wpdb->prefix . 'sat_message_historie WHERE chatID = %d', $id)); //db call ok
        wp_cache_set($cache_key, $items, 'sat-ai');
    }
    foreach ($items as &$item) {
        $item->message_time = get_date_from_gmt($item->message_time);
    }
    return $items;
}

/**
 * Deletes a single Chat Verlauf from database
 *
 * @param int   $id
 *
 * @return bool
 */
function sat_ai_delete_Chat_Verlauf($id = 0)
{
    if ($id == 0) {
        return false;
    }
    global $wpdb;

    $query1 = $wpdb->query($wpdb->prepare('DELETE FROM ' . $wpdb->prefix . 'sat_message_historie WHERE chatID = %d', $id)); //db call ok; no-cache ok
    $query2 = $wpdb->query($wpdb->prepare('DELETE FROM ' . $wpdb->prefix . 'sat_chathistorie WHERE chatID = %d', $id)); //db call ok; no-cache ok
    if ($query1 != false && $query2 != false) {
        return true;
    }
    return false;
}

function get_sat_ai_openai_role()
{
    $role_id = get_option("sat_openai_role");
    // Case: A role was never set
    if ($role_id == false) {
        return false;
    }
    $role_id = intval($role_id);
    if ($role_id == 0) {
        return false;
    } else {
        return get_post($role_id)->post_content;
    }
}

/**
 * Obscures a given string to just show the first 3-5 characters and last 3-5 characters
 *
 * @param string $sensitive_string
 *
 * @param int $strictness
 * Defines when the function should allow hints of the string
 *
 * @return string
 */
function sat_ai_obscure_sensitive_strings($sensitive_string, $strictness = 4)
{
    // assume we have an openai key/id
    $length_delta = 0;
    $sensitive_string_length = strlen($sensitive_string);
    $show_length = ceil(($sensitive_string_length / $strictness) - 1);
    if ($show_length > 5) {
        $show_length = 5;
    }

    // handle for short strings e.g. "1" --> "11"
    // or 123 --> 123123
    // API-Key
    if (str_starts_with($sensitive_string, "sk-")) {
        $length_delta = 3;
    // Assitant ID
    } elseif (str_starts_with($sensitive_string, "asst_")) {
        $length_delta = 5;
    }
    $first_five_characters = substr($sensitive_string, 0, $show_length + $length_delta);
    $last_five_characters = substr($sensitive_string, ($sensitive_string_length - $show_length >= 0 ? $sensitive_string_length - $show_length : 0), $show_length);
    $asterisk_count = $sensitive_string_length - strlen($first_five_characters) - strlen($last_five_characters);

    $obscured_string = $first_five_characters . str_repeat("*", $asterisk_count >= 0 ? $asterisk_count : 0) . $last_five_characters;
    return $obscured_string;
}

function sat_ai_get_tool_event_name($toolsname)
{
    // logic to retrive single eventname by tool-name/id #TODO implement when we use multiple tools
    return "sat_ai_contact";
}

function sat_ai_get_tool_event_names()
{
    // logic to retrieve all active tools and event names #TODO implement when we use multiple tools
    return array("sat_ai_contact");
}

function sat_ai_allowed_html()
{
    return array(
        'a' => array(
            'href' => array(),
            'title' => array()
        ),
        'br' => array(),
        'em' => array(),
        'strong' => array(),
        'ul' => array(),
        'li' => array(),
        'ol' => array(),
    );
}
